/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		Karl Guertin <grayrest@grayrest.com>
 *		James A. Overton <james@overton.ca>
 *		Christian Stocker (Bitflux)
 *		Paul Everitt
 *		David Palm
 *
 * ***** END LICENSE BLOCK ***** */

/**
@fileoverview
Mozile Toolbar Javascript.

Created by James A. Overton
Created on 2004-03-24

HISTORY:
	Some code in this file was first found in mozilehtmltb.js, while the 
majority was cut out of mozileTB.js and separated into different files.


DESCRIPTION:
	The code here deals with all of the active toolbar stuff, like removing
and replacing buttons and menus.

*/

function mozileHideToolbar() {mozileToolbar.collapsed=true}
function mozileShowToolbar() {mozileToolbar.collapsed=false}

/******** Check For Active Styles ********/
/*
function mozileCheckStyle(styleName, styleValue) {
	var active="false";

	var textContainer = mozile.lastContextNode;
	if(textContainer) { 
		var result=mozileTarget.document.defaultView.getComputedStyle(textContainer, null).getPropertyValue(styleName);
		if(result==styleValue) active="true";
	}

	return active;
}
*/
/*
Only works if style property is absolute.
Relative values are missed by getComputedStyle.

*/
function mozileCheckStyles(name, value, numberActive) {
	var s, v, cs;
	var active="false";
	try {
		var textContainer = mozile.lastContextNode;
		if(textContainer && numberActive>0) {
			for ( s=0; s<numberActive; s++ ) {
				cs = mozileTarget.document.defaultView.getComputedStyle(textContainer, null);
				v = cs.getPropertyValue(name[s]);
				if ( !mozileTarget.eDOM.compareStyleValues(name[s], value[s], v) )
					break;
			}
			if ( s==numberActive )
				active="true";
		}
	}catch(e){}
	return active;
}
/**
Checks if command is active.

@return "true" or "false"
@type String
*/
function mozileCheckActive(carr) {
	var active="false";
	var type = carr[6];
	var args = carr[7];

	switch(type) {
		case "toggleStyle":
			active=mozileCheckStyles(args[0],args[1],args[3]);
		break;

		case "toggleAttribute":
			var attrVal=mozile.lastContextNode.getAttribute(args[0]);
			if(args[1]==attrVal) active="true";
		break;
	}
	//alert(commandArray[0]+active);
	return active;
}

function mozileUpdateDisplay() {
	try{
		mozileUpdateButtons();
		mozileUpdateUndoButtons(true);
		mozileUpdateCommandMenu(true);
		mozileUpdateSource(true);
		mozileUpdateTagsButton(true);
	}catch(e){}
}

function mozileClearDisplay() {
	mozileClearButtons();
	mozileClearUndoButtons();
	mozileClearCommandMenu();
	mozileUpdateSource(true);
	mozileUpdateTagsButton(true)
}

/******** Command Menu ********/

function mozileUpdateCommandMenu(silent) {
	var f = ["mozileToolbar.js","mozileUpdateCommandMenu"];
	if(!silent) mozileDebug(f,3,"Updating Command Menu...");

	try {
		var contextTag = mozile.lastContextNode.tagName.toLowerCase();
		var tags = mozileTarget.document.mozileMES.tags;
		
		var tagIndex = mozileMES.findByName(contextTag, tags);
		if(tagIndex<0) {
			mozileDebug(f,1,"Context "+ contextTag +" is undefined!");
			mozileClearCommandMenu();	// clears the menu
			return; 
		}
		mozileMakeCommandMenu(tags[tagIndex][3]);
	} catch(e) {
		mozileDebug(f,0,"Exception: "+e);
	}
}

function mozileClearCommandMenu() {
	mozileCommandMenu.open=false; //Shouldn't be needed according to the specs but it keeps getting stuck open.
	mozileRemoveChildren(mozileCommandMenu);
	mozileCommandMenu.setAttribute("hidden", "true");
	return;
}

function mozileMakeCommandMenu(commandsArray) {
	if ( mozileCommandMenu.hasAttribute("disabled") && mozileCommandMenu.getAttribute("disabled")=="true" ) {
		mozileCommandMenu.setAttribute("hidden", "true");
	} else {
		mozileClearCommandMenu();
		if (mozileTarget) {
			var menupop = mozileMakeMenu(commandsArray);
			menupop.setAttribute("id","mozile-command-menu-popup");
			mozileCommandMenu.appendChild(menupop);
			mozileCommandMenu.setAttribute("hidden", "false");
		}
	}
}

/******** General Menus ********/

function mozileMakeMenu(commandsArray) {
	var menupopup = document.createElement('menupopup');
	for(var i=0; i<commandsArray.length; i++) {
		var menuitem = mozileMakeMenuitem(commandsArray[i]);
		if(menuitem!=0) { menupopup.appendChild(menuitem); }
	}
	return menupopup;
}


function mozileMakeMenuitem(commandArray) {
		// check to see if this command is in the mesCommands array
	var commands = mozileTarget.document.mozileMES.commands;
	var commandIndex = mozileMES.findByName(commandArray[0], commands);
		// if not, see if it's a commandGroup
	if(commandIndex<0) {
		if(commandArray.length!=7) {
			alert("Can't find command: "+commandArray[0]);
			return 0;
		}	
		
			// if it has the right shape, make a new 
		var garr = new Array();
		garr = commandArray;
		var menu = document.createElement('menu');
		menu.setAttribute('label',garr[1]);
		menu.setAttribute('accesskey',garr[4]);
		menu.setAttribute('tooltiptext',garr[2]);
		var newMenupopup = mozileMakeMenu(garr[6]);		// call this function recursively
		menu.appendChild(newMenupopup);
		return menu;
	}
	
	var carr = new Array();
	carr = commands[commandIndex];
	
	var menuitem = document.createElement('menuitem');
	menuitem.setAttribute('label',carr[1]);
	menuitem.setAttribute('accesskey',commandArray[1]);
	menuitem.setAttribute('tooltiptext',carr[2]);
	menuitem.setAttribute('oncommand','mozileFormat("'+ carr[0] +'")' ); // use the command name
		// if the command is currently active, then put a check next to the menuitem
	if(mozileCheckActive(carr)=="true") {
		menuitem.setAttribute("type","checkbox");
		menuitem.setAttribute("checked","true");
	}
	if ( carr[5] && carr[5]!="") {
		menuitem.setAttribute('key',carr[5]);
	}

	return menuitem;

}

/******** Toolbar Context Buttons ********/

function mozileClearButtons(silent) {
	var f = ["mozileToolbar.js","mozileClearButtons"];
	if(!silent) mozileDebug(f,3,"Clearing buttons...");
//	mozileArrowbox.scrollByIndex(-100);
	mozileRemoveChildren(mozileArrowbox);
	mozileUpdateToolbar(silent);
}

function mozileUpdateButtons(silent) {
	var f = ["mozileToolbar.js","mozileUpdateButtons"];
	if(!silent) mozileDebug(f,3,"Updating buttons...");

	mozileClearButtons(true);
	if (mozileTarget) {
		var mes = mozileTarget.document.mozileMES;
		var contextTag = mozile.lastContextNode.localName.toLowerCase();
		var tagIndex = mozileMES.findByName(contextTag, mes.tags);
		if(tagIndex<0) {
			mozileDebug(f,2,"Context undefined!"); 
		}else{
			mozileMakeButtons(mes.tags[tagIndex][3]);
		}
	}
}

/**
Disable and hide any of the fixed toolbar items that have been disabled via the MES.
Unhides any hidden buttons if no detection, otehrwise updates if MozileTarget
is valid, or if Mozile has been loaded.
Could be split into separate clear and update functions if required. 
*/
function mozileUpdateToolbar(silent) {
	var f = ["mozileToolbar.js","mozileUpdateToolbar"];
	if(!silent) mozileDebug(f,3,"Updating fixed toolbar buttons...");
	try {
		var t = mozileToolbar.getElementsByTagName("toolbarbutton");
		var mes = new Array();
	  
		if (mozileTarget) {
			mes = mozileTarget.document.mozileMES.toolbar;
		}else{
			var fWin = document.commandDispatcher.focusedWindow;
			if (mozileIsLoaded(fWin)) {
				mes = fWin.document.mozileMES.toolbar;
			}
		}
		//Set all the buttons to be shown
		for ( var b=0; b<t.length; b++ ) {
			var id = t[b].getAttribute("id");
			var found=false;
			for ( var m=0; m<mes.length; m++ ) {
				if ( id == "mozile-"+mes[m] ) {
					found=true;
					break;
				}
			}
			if ( found ) {			
				t[b].setAttribute("disabled", "true");
				t[b].setAttribute("hidden", "true");
			} else {
				if ( t[b].hasAttribute("disabled") )
					t[b].removeAttribute("disabled");
				if ( t[b].hasAttribute("hidden") )
					t[b].setAttribute("hidden", "false");
			}
		}
	}catch(e){
		mozileDebug(f,1,"Exception: "+e);
	}
}
/**
Finds the element with the specified id in the Mozile Toolbar.
Only looks at toolbarbuttons at the moment, but it could be extended to other elements.

@param {String} id : should begin with "mozile-..."
@return Element node with matching id, otherwise null
@type Element  
*/
function mozileGetToolbarById( id ) {
	var t = mozileToolbar.getElementsByTagName("toolbarbutton");
	for ( var b=0; b<t.length; b++ ) {
		if ( t[b].getAttribute("id") == id )
			return t[b];
	}
	return null;
}

/******** Source Editing ********/
/*
This is a temporary fix until source editing saves state for each editable window.
If the edit-source button has been disabled, and the source window is being displayed,
then switch it off.
*/
function mozileUpdateSource() {
	try {
		var button = document.getElementById("mozile-edit-source");
		if ( button.hasAttribute("active") && button.getAttribute("active")=="true"
			&& button.hasAttribute("disabled") && button.getAttribute("disabled")=="true" ) {
			mozileShowSourceView(); //Toggle source view off!
		}
	}catch(e){
		var f = ["mozileToolbar.js","mozileUpdateSource"];
		mozileDebug(f,1,"Exception updating Source Edit: "+e);
	}
}
/******** Toolbar Object ********/

var mozileTools = new MozileTools();

function MozileTools() {
	this.imageDir = "chrome://mozile/skin/buttons/";
	this.imageFormat = ".png";
	this.imageOff = "-disabled";
	this.imageUndo1 = this.imageDir+"undo"+this.imageFormat;
	this.imageRedo1 = this.imageDir+"redo"+this.imageFormat;
	this.imageUndo0 = this.imageDir+"undo"+this.imageOff+this.imageFormat;
	this.imageRedo0 = this.imageDir+"redo"+this.imageOff+this.imageFormat;
	this.clearUndoButtons = mozileClearUndoButtons;
	this.updateUndoButtons = mozileUpdateUndoButtons;
	this.clearTagsButton = mozileClearTagsButton;
	this.updateTagsButton = mozileUpdateTagsButton;
}
MozileTools.prototype.__defineGetter__ (
	"buttonUndo",
	function() {return document.getElementById("mozile-undo")}
);
MozileTools.prototype.__defineGetter__ (
	"buttonRedo",
	function() {return document.getElementById("mozile-redo")}
);
MozileTools.prototype.__defineGetter__ (
	"buttonTags",
	function() {return document.getElementById("mozile-view-tags")}
);

/******** Undo Redo Buttons ********/

function mozileClearUndoButtons() {
	mozileTools.buttonUndo.setAttribute( "image", mozileTools.imageUndo0 );
	mozileTools.buttonRedo.setAttribute( "image", mozileTools.imageRedo0 );
}

function mozileUpdateUndoButtons(silent) {
	var f = ["mozileTools.js","mozileUpdateUndoButtons"];
	if(!silent) mozileDebug(f,3,"Updating Undo Buttons...");
	try {
		if ( !mozileTarget || !("mozileUndo" in mozileTarget) )
			throw ("undo buttons off");
		var mUndo = mozileTarget.mozileUndo;
		mozileTools.buttonUndo.setAttribute("image", mUndo.isUndo ? mozileTools.imageUndo1 : mozileTools.imageUndo0);
		mozileTools.buttonRedo.setAttribute("image", mUndo.isRedo ? mozileTools.imageRedo1 : mozileTools.imageRedo0);
	} catch(e) { 
		mozileTools.clearUndoButtons();
	}
}

function mozileClearTagsButton() {
	mozileTools.buttonTags.setAttribute("active","false");
}

function mozileUpdateTagsButton(silent) {
	var f = ["mozileTools.js","mozileUpdateTagsButton"];
	if(!silent) mozileDebug(f,3,"Updating Tags Button...");
	try {
		var target = mozileGetCurrentTarget();
		if (!target) {
			target = document.commandDispatcher.focusedWindow;
			if ( !mozileIsLoaded( target ) ) {
				throw ("Not loaded");
			}
		}
		if ( !("mozileTags" in target) )
			throw ("Missing .mozileTags");
		var mTags = target.mozileTags;
		mTags.setActive( target, mTags.active );
	} catch(e) { 
		mozileTools.clearTagsButton();
	}
}

/******** General Buttons ********/
function mozileMakeButtons(commandsArray) {

	var button;
	for(var i=0; i<commandsArray.length; i++) {
			// only make a button if the button attribute is "true"
			// normal command case
		var mozileArrowbox = document.getElementById('mozile-arrowbox');
			
		var commandArray=commandsArray[i];
		if(commandArray.length==3 && commandArray[2]=="true") { 
			button = mozileMakeButton(commandArray);
			if(button!=0) { mozileArrowbox.appendChild(button); }
		}
			// command group case
		if(commandArray.length==7 && commandArray[5]=="true") { 
			button = mozileMakeButton(commandArray);
			if(button!=0) { mozileArrowbox.appendChild(button); }
		}
	}
}


function mozileMakeButton(commandArray) {
		// check to see if this command is in the mesCommands array
	var button, image;
	var commandIndex = mozileMES.findByName(commandArray[0], mozileTarget.document.mozileMES.commands);
		// if not, see if it's a commandGroup
	if(commandIndex<0) {
		if(commandArray.length!=7) {
			alert("Can't find command: "+commandArray[0]);
			return 0;
		}	
		
			// if it has the right shape, make a menu button
		var garr = new Array();
		garr = commandArray;
		
		if ( garr[3] )
			image = mozileTarget.document.mozileMES.images[0]+garr[3]+mozileTarget.document.mozileMES.images[2];
		
		button = document.createElement('toolbarbutton');
		button.id="mozile-"+garr[0]+"-menu";
		button.setAttribute('class','mozile-toolbar-menu');
		// Allow image or label for menu button, but not both.
		if ( image ) {
			button.setAttribute('image',image);
		} else { 
			button.setAttribute('label',garr[1]);
		}
		button.setAttribute('tooltiptext',garr[2])
		button.setAttribute('type','menu');
		var newMenupopup = mozileMakeMenu(garr[6]);			// call the menu creation function
		button.appendChild(newMenupopup);
		return button;
	}
	
	var carr = new Array();
	carr = mozileTarget.document.mozileMES.commands[commandIndex];
		
	image = mozileTarget.document.mozileMES.images[0]+carr[3]+mozileTarget.document.mozileMES.images[2];
	
	button = document.createElement('toolbarbutton');
	button.id="mozile-"+carr[0];
	button.setAttribute('class','mozile-toolbar-button');
	button.setAttribute('label',carr[1]);
	button.setAttribute('image',image);
	button.setAttribute('tooltiptext',carr[2]);
	button.setAttribute('active',mozileCheckActive(carr));
	button.setAttribute('oncommand','mozileFormat("'+ carr[0] +'")' ); 
	if ( carr[5] && carr[5]!="") {
		button.setAttribute('key',carr[5]);
	}

	return button;
}
